#!/usr/bin/python3
# coding: utf8

# The Incredible Machine (TIM) level/image resource viewer, version 0.2
# "Extra" module, not covered by the CC0 license of "timgres" package

from __future__ import division, print_function, unicode_literals

import itertools, sys

if sys.version_info[0] != 3:
    raise NotImplementedError('Only Python 3 is supported.')
    # izip = itertools.izip
else:
    izip = zip
    xrange = range

# --- Multi-(sub-)-image animation frame information for TIM/TEMIM ---

# NOTE: In TIM 2 and 3, positions of the bitmap sub-images for animation
#     frames are stored in the .ANM file. However, no sub-file of the
#     TIM/TEMIM resource file seem to contain such information, it is very
#     likely stored in the .EXE file.
#     As I have not figured out this yet, an "alternative" solution is
#     provided here: pixel exact "reconstruction" based on screenshots
#     and the converted sub_images

# Intentionally does not import vdobase or any other module

flip_none = 0

# Constants indicating whether the frame is the first in an animation, etc.
anim_unused = 0
anim_first_frame = 1
anim_continue = 2
anim_first_flipped = 3

# tim_temim_info_checksums: The check-sums used for identifying the game
#     version.
# A tuple, one item for each group (depending on presence/absence in)
#     versions
# Format if "inner" tuples:
#     0. group id (based on availability in demo/full game), int
#     1. image resource name (str)
#     2. the crc32 checksum (unsigned 32 bit int)
#     Items with a negative number as their first item have a special meaning,
#     see puzzle/puzdata.py

tim_temim_info_checksums = (
    (0, 'PART0.BMP', 0x883d3367),
    (1, 'PART17.BMP', 0x5a0825f7),
    (2, 'PART52.BMP', 0x2b00b572),
    (-1, 'PART10.BMP', 0x2842ecd0),
    )

# tim_temim_frame_sub_image_pos: The positions (x, y) & other important data
#     for each animation frame
# Format of inner tuples:
#     0. Frame information tuple, see format below
#     1... information for each sub-image that will be displayed
#
# Format of "inner-inner" tuple 0 (frame information)
#     0. The group id (from info_checksums)
#     1. animation information
#     2. frame width
#     3. frame height
#     4. x offset (frame)
#     5. y offset (frame)
#
# Format of "inner-inner" tuples 1... (info for each sub-image):
#     0. Resource file I.D eg. 15 -> 'PART15.BMP'
#     1. Resource sub-item ID
#     2. x offset
#     3. y offset
#     4. reserved for flip, currently unused

tim_temim_frame_sub_image_pos = (
    # ... Part 13: Jack-in-the-box ...
    (
        (1, anim_first_frame, 43, 91, 6, 59),
        (13, 0, 0, 3, flip_none),
        (13, 1, 0, 0, flip_none),
        (13, 2, 9, 9, flip_none),
        ),
    (
        (1, anim_continue, 43, 91, 6, 59),
        (13, 0, 0, 3, flip_none),
        (13, 1, 0, 0, flip_none),
        (13, 3, 9, 9, flip_none),
        ),
    (
        (1, anim_continue, 43, 91, 6, 59),
        (13, 0, 0, 3, flip_none),
        (13, 1, 0, 0, flip_none),
        (13, 4, 9, 9, flip_none),
        ),
    (
        (1, anim_continue, 43, 91, 6, 59),
        (13, 0, 0, 3, flip_none),
        (13, 1, 0, 0, flip_none),
        (13, 5, 9, 9, flip_none),
        ),
    (
        (1, anim_continue, 43, 91, 6, 59),
        (13, 0, 0, 3, flip_none),
        (13, 1, 0, 0, flip_none),
        (13, 6, 9, 9, flip_none),
        ),
    (
        (1, anim_continue, 43, 91, 6, 59),
        (13, 0, 0, 3, flip_none),
        (13, 1, 0, 0, flip_none),
        (13, 7, 8, 9, flip_none),
        ),
    (
        (1, anim_continue, 43, 91, 6, 59),
        (13, 0, 0, 3, flip_none),
        (13, 1, 0, 0, flip_none),
        (13, 8, 9, 9, flip_none),
        ),
    (
        (1, anim_continue, 43, 91, 6, 59),
        (13, 0, 0, 3, flip_none),
        (13, 1, 0, 0, flip_none),
        (13, 9, 9, 8, flip_none),
        ),
    (
        (1, anim_first_frame, 43, 91, 6, 38),
        (13, 0, 0, 24, flip_none),
        (13, 2, 9, 30, flip_none),
        (13, 10, 0, 0, flip_none),
        ),
    (
        (1, anim_continue, 43, 91, 6, 25),
        (13, 0, 0, 37, flip_none),
        (13, 2, 9, 43, flip_none),
        (13, 11, 1, 0, flip_none),
        ),
    (
        (1, anim_continue, 43, 91, 0, 0),
        (13, 0, 6, 62, flip_none),
        (13, 2, 15, 68, flip_none),
        (13, 12, 0, 0, flip_none),
        ),
    (
        (1, anim_continue, 43, 91, 2, 34),
        (13, 0, 4, 28, flip_none),
        (13, 2, 13, 34, flip_none),
        (13, 13, 0, 0, flip_none),
        ),
    (
        (1, anim_continue, 43, 91, 0, 29),
        (13, 0, 6, 33, flip_none),
        (13, 2, 15, 39, flip_none),
        (13, 14, 0, 0, flip_none),
        ),
    (
        (1, anim_continue, 43, 91, 2, 34),
        (13, 0, 4, 28, flip_none),
        (13, 2, 13, 34, flip_none),
        (13, 15, 0, 0, flip_none),
        ),
    (
        (1, anim_continue, 43, 91, 2, 34),
        (13, 0, 4, 28, flip_none),
        (13, 2, 13, 34, flip_none),
        (13, 16, 0, 0, flip_none),
        ),
    (
        (1, anim_continue, 43, 91, 2, 34),
        (13, 0, 4, 28, flip_none),
        (13, 2, 13, 34, flip_none),
        (13, 17, 0, 0, flip_none),
        ),
    (
        (1, anim_continue, 43, 91, 2, 34),
        (13, 0, 4, 28, flip_none),
        (13, 2, 13, 34, flip_none),
        (13, 18, 0, 0, flip_none),
        ),
    (
        (1, anim_continue, 43, 91, 2, 34),
        (13, 0, 4, 28, flip_none),
        (13, 2, 13, 34, flip_none),
        (13, 19, 0, 0, flip_none),
        ),
    (
        (1, anim_continue, 43, 91, 2, 34),
        (13, 0, 4, 28, flip_none),
        (13, 2, 13, 34, flip_none),
        (13, 20, 0, 0, flip_none),
        ),
    # ... Part 15: Aquarium ...
    (
        (1, anim_first_frame, 105, 57, 28, 0),
        (15, 0, 0, 0, flip_none),
        (15, 1, 10, 11, flip_none),
        (15, 12, 10, 27, flip_none),
        ),
    (
        (1, anim_continue, 105, 57, 28, 0),
        (15, 0, 0, 0, flip_none),
        (15, 2, 15, 9, flip_none),
        (15, 12, 10, 27, flip_none),
        ),
    (
        (1, anim_continue, 105, 57, 28, 0),
        (15, 0, 0, 0, flip_none),
        (15, 3, 26, 16, flip_none),
        (15, 12, 10, 27, flip_none),
        ),
    (
        (1, anim_continue, 105, 57, 28, 0),
        (15, 0, 0, 0, flip_none),
        (15, 4, 31, 17, flip_none),
        (15, 12, 10, 27, flip_none),
        ),
    (
        (1, anim_continue, 105, 57, 28, 0),
        (15, 0, 0, 0, flip_none),
        (15, 5, 26, 16, flip_none),
        (15, 12, 10, 27, flip_none),
        ),
    (
        (1, anim_continue, 105, 57, 28, 0),
        (15, 0, 0, 0, flip_none),
        (15, 6, 10, 11, flip_none),
        (15, 12, 10, 27, flip_none),
        ),
    (
        (1, anim_continue, 105, 57, 28, 0),
        (15, 0, 0, 0, flip_none),
        (15, 7, 7, 11, flip_none),
        (15, 12, 10, 27, flip_none),
        ),
    (
        (1, anim_continue, 105, 57, 28, 0),
        (15, 0, 0, 0, flip_none),
        (15, 8, 6, 11, flip_none),
        (15, 12, 10, 27, flip_none),
        ),
    (
        (1, anim_continue, 105, 57, 28, 0),
        (15, 0, 0, 0, flip_none),
        (15, 9, 7, 16, flip_none),
        (15, 12, 10, 27, flip_none),
        ),
    (
        (1, anim_continue, 105, 57, 28, 0),
        (15, 0, 0, 0, flip_none),
        (15, 10, 8, 16, flip_none),
        (15, 12, 10, 27, flip_none),
        ),
    (
        (1, anim_continue, 105, 57, 28, 0),
        (15, 0, 0, 0, flip_none),
        (15, 11, 6, 10, flip_none),
        (15, 12, 10, 27, flip_none),
        ),
    (
        (1, anim_first_frame, 105, 57, 12, 8),
        (15, 13, 0, 0, flip_none),
        ),
    (
        (1, anim_continue, 105, 57, 9, 15),
        (15, 14, 0, 0, flip_none),
        ),
    (
        (1, anim_continue, 105, 57, 9, 15),
        (15, 15, 0, 0, flip_none),
        ),
    (
        (1, anim_continue, 105, 57, 0, 25),
        (15, 16, 0, 0, flip_none),
        (15, 17, 33, 14, flip_none),
        ),
    (
        (1, anim_continue, 105, 57, 0, 25),
        (15, 16, 0, 0, flip_none),
        (15, 18, 33, 6, flip_none),
        ),
    (
        (1, anim_continue, 105, 57, 0, 25),
        (15, 16, 0, 0, flip_none),
        (15, 19, 31, 4, flip_none),
        ),
    (
        (1, anim_continue, 105, 57, 0, 25),
        (15, 16, 0, 0, flip_none),
        (15, 20, 33, 4, flip_none),
        ),
    (
        (1, anim_continue, 105, 57, 0, 25),
        (15, 16, 0, 0, flip_none),
        (15, 21, 23, 2, flip_none),
        ),
    (
        (1, anim_continue, 105, 57, 0, 22),
        (15, 16, 0, 3, flip_none),
        (15, 22, 23, 0, flip_none),
        ),
    (
        (1, anim_continue, 105, 57, 0, 25),
        (15, 16, 0, 0, flip_none),
        (15, 23, 29, 6, flip_none),
        ),
    (
        (1, anim_continue, 105, 57, 0, 25),
        (15, 16, 0, 0, flip_none),
        (15, 24, 33, 11, flip_none),
        ),
    (
        (1, anim_continue, 105, 57, 0, 25),
        (15, 16, 0, 0, flip_none),
        (15, 25, 33, 13, flip_none),
        ),
    # ... Part 18: Cannon ...
    (
        (1, anim_first_frame, 214, 98, 9, 46),
        (18, 0, 0, 0, flip_none),
        (18, 9, 9, 13, flip_none),
        ),
    (
        (1, anim_continue, 214, 98, 0, 40),
        (18, 0, 9, 6, flip_none),
        (18, 9, 18, 19, flip_none),
        (18, 10, 0, 0, flip_none),
        ),
    (
        (1, anim_continue, 214, 98, 0, 40),
        (18, 0, 9, 6, flip_none),
        (18, 9, 18, 19, flip_none),
        (18, 11, 0, 0, flip_none),
        ),
    (
        (1, anim_continue, 214, 98, 3, 41),
        (18, 0, 6, 5, flip_none),
        (18, 9, 15, 18, flip_none),
        (18, 12, 0, 0, flip_none),
        ),
    (
        (1, anim_continue, 214, 98, 2, 43),
        (18, 0, 7, 3, flip_none),
        (18, 9, 16, 16, flip_none),
        (18, 13, 0, 0, flip_none),
        ),
    (
        (1, anim_continue, 214, 98, 3, 44),
        (18, 0, 6, 2, flip_none),
        (18, 9, 15, 15, flip_none),
        (18, 14, 0, 0, flip_none),
        ),
    (
        (1, anim_continue, 214, 98, 2, 38),
        (18, 1, 0, 0, flip_none),
        (18, 9, 16, 21, flip_none),
        ),
    (
        (1, anim_continue, 214, 98, 1, 35),
        (18, 2, 0, 0, flip_none),
        (18, 9, 17, 24, flip_none),
        ),
    (
        (1, anim_continue, 214, 98, 7, 14),
        (18, 3, 0, 29, flip_none),
        (18, 4, 85, 0, flip_none),
        (18, 9, 11, 45, flip_none),
        ),
    (
        (1, anim_continue, 214, 98, 7, 0),
        (18, 5, 0, 39, flip_none),
        (18, 6, 103, 0, flip_none),
        (18, 9, 11, 59, flip_none),
        ),
    (
        (1, anim_continue, 214, 98, 6, 13),
        (18, 7, 0, 30, flip_none),
        (18, 8, 130, 0, flip_none),
        (18, 9, 12, 46, flip_none),
        ),
    (
        (1, anim_continue, 214, 98, 9, 46),
        (18, 0, 0, 0, flip_none),
        (18, 9, 9, 13, flip_none),
        (18, 15, 0, 2, flip_none),
        ),
    # ... Part 19: Dynamite ...
    (
        (1, anim_first_frame, 55, 28, 0, 0),
        (19, 0, 0, 0, flip_none),
        ),
    (
        (1, anim_continue, 55, 28, 0, 0),
        (19, 0, 0, 0, flip_none),
        (19, 1, 40, 9, flip_none),
        ),
    (
        (1, anim_continue, 55, 28, 0, 0),
        (19, 0, 0, 0, flip_none),
        (19, 2, 39, 10, flip_none),
        ),
    (
        (1, anim_continue, 55, 28, 0, 0),
        (19, 0, 0, 0, flip_none),
        (19, 3, 39, 9, flip_none),
        ),
    (
        (1, anim_continue, 55, 28, 0, 0),
        (19, 0, 0, 0, flip_none),
        (19, 4, 37, 13, flip_none),
        ),
    (
        (1, anim_continue, 55, 28, 0, 0),
        (19, 0, 0, 0, flip_none),
        (19, 5, 38, 12, flip_none),
        ),
    # ... Part 21: Electric outlet with switch ...
    (
        (1, anim_first_frame, 48, 32, 0, 0),
        (21, 0, 0, 0, flip_none),
        (21, 1, 8, 8, flip_none),
        ),
    (
        (1, anim_first_frame, 48, 32, 0, 0),
        (21, 0, 0, 0, flip_none),
        (21, 1, 8, 8, flip_none),
        (21, 3, 29, 4, flip_none),

        ),
    (
        (1, anim_first_frame, 48, 32, 0, 0),
        (21, 0, 0, 0, flip_none),
        (21, 1, 8, 8, flip_none),
        (21, 3, 29, 18, flip_none),
        ),
    (
        (1, anim_first_frame, 48, 32, 0, 0),
        (21, 0, 0, 0, flip_none),
        (21, 1, 8, 8, flip_none),
        (21, 3, 29, 4, flip_none),
        (21, 3, 29, 18, flip_none),
        ),
    (
        (1, anim_first_frame, 48, 32, 0, 0),
        (21, 0, 0, 0, flip_none),
        (21, 2, 8, 8, flip_none),
        ),
    (
        (1, anim_first_frame, 48, 32, 0, 0),
        (21, 0, 0, 0, flip_none),
        (21, 2, 8, 8, flip_none),
        (21, 3, 29, 4, flip_none),

        ),
    (
        (1, anim_first_frame, 48, 32, 0, 0),
        (21, 0, 0, 0, flip_none),
        (21, 2, 8, 8, flip_none),
        (21, 3, 29, 18, flip_none),
        ),
    (
        (1, anim_first_frame, 48, 32, 0, 0),
        (21, 0, 0, 0, flip_none),
        (21, 2, 8, 8, flip_none),
        (21, 3, 29, 4, flip_none),
        (21, 3, 29, 18, flip_none),
        ),
    # ... Part 22: Dynamite & Plunger ...
    (
        (1, anim_first_frame, 135, 47, 0, 0),
        (22, 2, 103, 0, flip_none),
        (22, 0, 0, 19, flip_none),
        (22, 1, 40, 16, flip_none),
        ),
    (
        (1, anim_continue, 135, 47, 0, 5),
        (22, 2, 103, 0, flip_none),
        (22, 0, 0, 14, flip_none),
        (22, 1, 40, 11, flip_none),
        ),
    (
        (1, anim_continue, 135, 47, 40, 10),
        (22, 2, 63, 0, flip_none),
        (22, 1, 0, 6, flip_none),
        ),
    # ... Part 24: Electric fan ...
    (
        (1, anim_first_frame, 32, 32, 0, 0),
        (24, 0, 0, 8, flip_none),
        (24, 1, 16, 0, flip_none),
        ),
    (
        (1, anim_continue, 32, 32, 0, 0),
        (24, 0, 0, 8, flip_none),
        (24, 2, 16, 0, flip_none),

        ),
    (
        (1, anim_continue, 32, 32, 0, 0),
        (24, 0, 0, 8, flip_none),
        (24, 3, 16, 0, flip_none),
        ),
    (
        (1, anim_continue, 32, 32, 0, 0),
        (24, 0, 0, 8, flip_none),
        (24, 4, 16, 0, flip_none),
        ),
    # ... Part 26: Generator ...
    (
        (1, anim_first_frame, 72, 38, 0, 6),
        (26, 0, 0, 0, flip_none),
        (26, 1, 21, 0, flip_none),
        ),
    (
        (1, anim_first_frame, 72, 38, 0, 6),
        (26, 0, 0, 0, flip_none),
        (26, 1, 21, 0, flip_none),
        (26, 5, 5, 4, flip_none),
        ),
    (
        (1, anim_first_frame, 72, 38, 0, 6),
        (26, 1, 21, 0, flip_none),
        (26, 0, 0, 0, flip_none),
        (26, 5, 5, 18, flip_none),
        ),
    (
        (1, anim_first_frame, 72, 38, 0, 6),
        (26, 0, 0, 0, flip_none),
        (26, 1, 21, 0, flip_none),
        (26, 5, 5, 4, flip_none),
        (26, 5, 5, 18, flip_none),
        ),
    (
        (1, anim_first_frame, 72, 38, 0, 6),
        (26, 0, 0, 0, flip_none),
        (26, 1, 21, 0, flip_none),
        (26, 5, 5, 4, flip_none),
        ),
    (
        (1, anim_continue, 72, 38, 0, 0),
        (26, 0, 0, 6, flip_none),
        (26, 2, 21, 0, flip_none),
        (26, 5, 5, 10, flip_none),
        ),
    (
        (1, anim_continue, 72, 38, 0, 6),
        (26, 0, 0, 0, flip_none),
        (26, 3, 21, 1, flip_none),
        (26, 5, 5, 4, flip_none),

        ),
    (
        (1, anim_continue, 72, 38, 0, 1),
        (26, 0, 0, 5, flip_none),
        (26, 4, 21, 0, flip_none),
        (26, 5, 5, 9, flip_none),
        ),
    # ... Part 27: Gun ...
    (
        (1, anim_first_frame, 139, 46, 15, 12),
        (27, 0, 0, 0, flip_none),
        ),
    (
        (1, anim_continue, 139, 46, 13, 0),
        (27, 0, 0, 12, flip_none),
        (27, 5, 66, 0, flip_none),
        ),
    (
        (1, anim_continue, 139, 46, 14, 7),
        (27, 1, 0, 0, flip_none),
        ),
    (
        (1, anim_continue, 139, 46, 13, 9),
        (27, 2, 0, 0, flip_none),
        ),
    (
        (1, anim_continue, 139, 46, 0, 6),
        (27, 3, 0, 0, flip_none),
        ),
    (
        (1, anim_continue, 139, 46, 4, 9),
        (27, 4, 0, 0, flip_none),
        ),
    # ... Part 29: Light bulb ...
    # Modified: sub-items 2 and 3 moved downwards by 18 pixels
    (
        (1, anim_first_frame, 47, 82, 8, 18),
        (29, 0, 0, 0, flip_none),
        (29, 4, 20, 28, flip_none),

        ),
    (
        (1, anim_first_frame, 47, 82, 0, 0),
        (29, 1, 0, 0, flip_none),
        (29, 5, 28, 46, flip_none),
        ),
    (
        (1, anim_first_flipped, 47, 50, 8, 18),
        (29, 2, 0, 0, flip_none),
        (29, 4, 19, 2, flip_none),

        ),
    (
        (1, anim_first_frame, 47, 50, 0, 18),
        (29, 3, 0, 0, flip_none),
        (29, 5, 27, 2, flip_none),
        ),
    # ... Part 31: Monkey motor ---
    (
        (0, anim_first_frame, 92, 79, 0, 0),
        (31, 4, 40, 0, flip_none),
        (31, 0, 0, 12, flip_none),
        ),
    (
        (0, anim_first_frame, 92, 79, 0, 6),
        (31, 5, 40, 0, flip_none),
        (31, 0, 0, 6, flip_none),
        ),
    (
        (0, anim_continue, 92, 79, 0, 6),
        (31, 5, 40, 0, flip_none),
        (31, 1, 0, 6, flip_none),
        ),
    (
        (0, anim_continue, 92, 79, 0, 6),
        (31, 5, 40, 0, flip_none),
        (31, 2, 0, 6, flip_none),
        ),
    (
        (0, anim_continue, 92, 79, 0, 6),
        (31, 5, 40, 0, flip_none),
        (31, 3, 0, 6, flip_none),
        ),
    (
        (0, anim_first_frame, 92, 79, 0, 0),
        (31, 6, 14, 1, flip_none),
        (31, 5, 40, 6, flip_none),
        (31, 0, 0, 12, flip_none),
        ),
    (
        (0, anim_continue, 92, 79, 0, 0),
        (31, 7, 16, 1, flip_none),
        (31, 5, 40, 6, flip_none),
        (31, 0, 0, 12, flip_none),
        ),
    (
        (0, anim_continue, 92, 79, 0, 0),
        (31, 8, 16, 1, flip_none),
        (31, 5, 40, 6, flip_none),
        (31, 0, 0, 12, flip_none),
        ),
    (
        (0, anim_continue, 92, 79, 0, 0),
        (31, 9, 15, 1, flip_none),
        (31, 5, 40, 6, flip_none),
        (31, 0, 0, 12, flip_none),
        ),
    # ****
    (
        (0, anim_first_frame, 92, 79, 0, 0),
        (31, 6, 14, 1, flip_none),
        (31, 4, 40, 0, flip_none),
        (31, 0, 0, 12, flip_none),
        ),
    (
        (0, anim_continue, 92, 79, 0, 0),
        (31, 7, 16, 1, flip_none),
        (31, 4, 40, 0, flip_none),
        (31, 0, 0, 12, flip_none),
        ),
    (
        (0, anim_continue, 92, 79, 0, 0),
        (31, 8, 16, 1, flip_none),
        (31, 4, 40, 0, flip_none),
        (31, 0, 0, 12, flip_none),
        ),
    (
        (0, anim_continue, 92, 79, 0, 0),
        (31, 9, 15, 1, flip_none),
        (31, 4, 40, 0, flip_none),
        (31, 0, 0, 12, flip_none),
        ),

    # ... Part 36: Rocket ...
    (
        (1, anim_first_frame, 19, 84, 2, 3),
        (36, 0, 0, 0, flip_none),
        (36, 1, 3, 45, flip_none),

        ),
    (
        (1, anim_continue, 19, 84, 2, 3),
        (36, 0, 0, 0, flip_none),
        (36, 2, 2, 45, flip_none),

        ),
    (
        (1, anim_continue, 19, 84, 2, 3),
        (36, 0, 0, 0, flip_none),
        (36, 3, 2, 45, flip_none),

        ),
    (
        (1, anim_continue, 19, 84, 2, 3),
        (36, 0, 0, 0, flip_none),
        (36, 4, 0, 45, flip_none),

        ),
    (
        (1, anim_continue, 19, 84, 0, 3),
        (36, 0, 2, 0, flip_none),
        (36, 5, 0, 45, flip_none),

        ),
    (
        (1, anim_continue, 19, 84, 0, 3),
        (36, 0, 2, 0, flip_none),
        (36, 6, 0, 45, flip_none),

        ),
    (
        (1, anim_continue, 19, 84, 2, 3),
        (36, 0, 0, 0, flip_none),
        (36, 7, 0, 44, flip_none),

        ),
    (
        (1, anim_continue, 19, 84, 2, 3),
        (36, 0, 0, 0, flip_none),
        (36, 8, 1, 46, flip_none),

        ),
    (
        (1, anim_continue, 19, 84, 2, 3),
        (36, 0, 0, 0, flip_none),
        (36, 9, 0, 46, flip_none),
        ),
    (
        (1, anim_continue, 19, 84, 2, 0),
        (36, 0, 0, 0, flip_none),
        (36, 10, 0, 46, flip_none),

        ),
    # ... Part 37: Scissors ...
    (
        (1, anim_first_frame, 42, 34, 2, 0),
        (37, 0, 0, 0, flip_none),
        (37, 1, 21, 17, flip_none),
        ),
    (
        (1, anim_first_frame, 42, 34, 0, 4),
        (37, 2, 0, 0, flip_none),
        ),
    # ... Part 38: Solar panel ...
    (
        (1, anim_first_frame, 70, 32, 0, 0),
        (38, 0, 0, 0, flip_none),

        ),
    (
        (1, anim_first_frame, 70, 32, 0, 0),
        (38, 0, 0, 0, flip_none),
        (38, 1, 52, 4, flip_none),

        ),
    (
        (1, anim_first_frame, 70, 32, 0, 0),
        (38, 0, 0, 0, flip_none),
        (38, 1, 52, 18, flip_none),
        ),
    (
        (1, anim_first_frame, 70, 32, 0, 0),
        (38, 0, 0, 0, flip_none),
        (38, 1, 52, 4, flip_none),
        (38, 1, 52, 18, flip_none),
        ),
    # ... Part 39: Springboard ...
    (
        (1, anim_first_frame, 48, 31, 0, 3),
        (39, 0, 0, 0, flip_none),
        (39, 1, 0, 9, flip_none),
        ),
    (
        (1, anim_continue, 48, 31, 0, 3),
        (39, 2, 0, 0, flip_none),
        (39, 3, 0, 9, flip_none),
        ),
    (
        (1, anim_continue, 48, 31, 0, 3),
        (39, 4, 0, 0, flip_none),
        (39, 5, 0, 9, flip_none),
        ),
    (
        (1, anim_continue, 48, 31, 0, 3),
        (39, 6, 0, 0, flip_none),
        (39, 1, 0, 9, flip_none),
        ),
    (
        (1, anim_continue, 48, 31, 0, 0),
        (39, 7, 0, 0, flip_none),
        (39, 1, 0, 12, flip_none),
        ),
    # ... Part 45: Candle ...
    (
        (1, anim_first_frame, 40, 36, 0, 0),
        (45, 0, 0, 4, flip_none),
        (45, 1, 11, 0, flip_none),
        ),
    (
        (1, anim_continue, 40, 36, 0, 0),
        (45, 0, 0, 4, flip_none),
        (45, 2, 11, 0, flip_none),
        ),
    (
        (1, anim_continue, 40, 36, 0, 0),
        (45, 0, 0, 4, flip_none),
        (45, 3, 11, 0, flip_none),
        ),
    (
        (1, anim_continue, 40, 36, 0, 0),
        (45, 0, 0, 4, flip_none),
        (45, 4, 11, 0, flip_none),
        ),
    (
        (1, anim_continue, 40, 36, 0, 0),
        (45, 0, 0, 4, flip_none),
        (45, 5, 11, 0, flip_none),
        ),
    # ... Part 51: Vacuum ...
    (
        (2, anim_first_frame, 66, 51, 3, 4),
        (51, 5, 7, 9, flip_none),
        (51, 3, 0, 0, flip_none),
        (51, 0, 17, 32, flip_none),
        ),
    (
        (2, anim_continue, 66, 51, 3, 0),
        (51, 6, 3, 11, flip_none),
        (51, 3, 0, 0, flip_none),
        (51, 1, 18, 32, flip_none),
        (51, 4, 44, 35, flip_none),
        ),
    (
        (2, anim_continue, 66, 51, 3, 4),
        (51, 7, 0, 9, flip_none),
        (51, 3, 0, 0, flip_none),
        (51, 0, 17, 32, flip_none),
        (51, 4, 47, 35, flip_none),
        ),
    (
        (2, anim_continue, 66, 51, 0, 5),
        (51, 8, 0, 10, flip_none),
        (51, 3, 3, 0, flip_none),
        (51, 2, 18, 33, flip_none),
        (51, 4, 52, 36, flip_none),
        ),
    (
        (2, anim_first_frame, 66, 51, 0, 4),
        (51, 9, 0, 8, flip_none),
        (51, 3, 3, 0, flip_none),
        (51, 0, 20, 32, flip_none),
        (51, 4, 50, 35, flip_none),
        ),
    (
        (2, anim_continue, 66, 51, 1, 0),
        (51, 9, 0, 9, flip_none),
        (51, 3, 2, 0, flip_none),
        (51, 1, 20, 32, flip_none),
        (51, 4, 46, 35, flip_none),
        ),
    (
        (2, anim_continue, 66, 51, 0, 5),
        (51, 9, 0, 8, flip_none),
        (51, 3, 3, 0, flip_none),
        (51, 2, 18, 33, flip_none),
        (51, 4, 52, 36, flip_none),
        ),
    # ... Part 58: Mel's house ...
    (
        (2, anim_first_frame, 66, 97, 18, 33),
        (58, 0, 0, 0, flip_none),
        (58, 1, 28, 39, flip_none),
        (58, 2, 8, 39, flip_none),

        ),
    (
        (2, anim_continue, 66, 97, 18, 33),
        (58, 0, 0, 0, flip_none),
        (58, 1, 28, 39, flip_none),
        (58, 3, 8, 39, flip_none),
        ),
    (
        (2, anim_continue, 66, 97, 18, 33),
        (58, 0, 0, 0, flip_none),
        (58, 1, 28, 39, flip_none),
        (58, 4, 8, 39, flip_none),
        ),
    (
        (2, anim_continue, 66, 97, 18, 33),
        (58, 0, 0, 0, flip_none),
        (58, 1, 28, 39, flip_none),
        (58, 5, 8, 39, flip_none),
        ),
    (
        (2, anim_first_frame, 66, 97, 6, 5),
        (58, 0, 12, 28, flip_none),
        (58, 1, 40, 67, flip_none),
        (58, 5, 20, 67, flip_none),
        (58, 6, 0, 0, flip_none),
        ),
    (
        (2, anim_continue, 66, 97, 1, 3),
        (58, 0, 17, 30, flip_none),
        (58, 1, 45, 69, flip_none),
        (58, 5, 25, 69, flip_none),
        (58, 7, 0, 0, flip_none),
        ),
    (
        (2, anim_continue, 66, 97, 0, 0),
        (58, 0, 18, 33, flip_none),
        (58, 1, 46, 72, flip_none),
        (58, 5, 26, 72, flip_none),
        (58, 8, 0, 0, flip_none),
        ),
    # ... Part 61: Alligator ...
    # Note: Sub-image 9 is not found here as it cannot
    #     be observed (?) in the game
    (
        (2, anim_first_frame, 87, 33, 0, 17),
        (61, 0, 37, 0, flip_none),
        (61, 1, 0, 0, flip_none),
        (61, 10, 62, 0, flip_none),
        ),
    (
        (2, anim_continue, 87, 33, 0, 17),
        (61, 0, 37, 0, flip_none),
        (61, 1, 0, 0, flip_none),
        (61, 11, 62, 2, flip_none),
        ),
    (
        (2, anim_continue, 87, 33, 0, 17),
        (61, 0, 37, 0, flip_none),
        (61, 1, 0, 0, flip_none),
        (61, 12, 62, 2, flip_none),
        ),
    (
        (2, anim_continue, 87, 33, 0, 17),
        (61, 0, 37, 0, flip_none),
        (61, 1, 0, 0, flip_none),
        (61, 13, 62, 2, flip_none),
        ),
    (
        (2, anim_continue, 87, 33, 0, 17),
        (61, 0, 37, 0, flip_none),
        (61, 1, 0, 0, flip_none),
        (61, 14, 62, 1, flip_none),
        ),
    (
        (2, anim_continue, 87, 33, 0, 17),
        (61, 0, 37, 0, flip_none),
        (61, 1, 0, 0, flip_none),
        (61, 15, 62, 2, flip_none),
        ),
    (
        (2, anim_first_frame, 87, 33, 2, 7),
        (61, 0, 35, 10, flip_none),
        (61, 2, 0, 0, flip_none),
        (61, 10, 60, 10, flip_none),
        ),
    (
        (2, anim_continue, 87, 33, 4, 0),
        (61, 0, 33, 17, flip_none),
        (61, 3, 0, 0, flip_none),
        (61, 10, 58, 17, flip_none),
        ),
    (
        (2, anim_first_frame, 87, 33, 0, 17),
        (61, 0, 37, 0, flip_none),
        (61, 4, 0, 0, flip_none),
        (61, 10, 62, 0, flip_none),
        ),
    (
        (2, anim_continue, 87, 33, 0, 17),
        (61, 0, 37, 0, flip_none),
        (61, 5, 0, 0, flip_none),
        (61, 10, 62, 0, flip_none),
        ),
    (
        (2, anim_continue, 87, 33, 0, 17),
        (61, 0, 37, 0, flip_none),
        (61, 6, 0, 0, flip_none),
        (61, 10, 62, 0, flip_none),
        ),
    (
        (2, anim_continue, 87, 33, 0, 17),
        (61, 0, 37, 0, flip_none),
        (61, 7, 0, 0, flip_none),
        (61, 10, 62, 0, flip_none),
        ),
    (
        (2, anim_continue, 87, 33, 0, 17),
        (61, 0, 37, 0, flip_none),
        (61, 8, 0, 0, flip_none),
        (61, 10, 62, 0, flip_none),
        ),
    # ... Part 62: Teapot ...
    # Note: Sub-image 3 is not found here as it cannot
    #     be observed (?) in the game
    (
        (2, anim_first_frame, 88, 57, 45, 35),
        (62, 0, 0, 0, flip_none),
        ),
    (
        (2, anim_continue, 88, 57, 47, 31),
        (62, 1, 0, 0, flip_none),
        ),
    (
        (2, anim_continue, 88, 57, 48, 22),
        (62, 2, 0, 0, flip_none),
        ),
    (
        (2, anim_first_frame, 88, 57, 0, 1),
        (62, 4, 41, 34, flip_none),
        (62, 6, 0, 0, flip_none),
        ),
    (
        (2, anim_continue, 88, 57, 2, 1),
        (62, 4, 39, 34, flip_none),
        (62, 7, 0, 0, flip_none),
        ),
    (
        (2, anim_continue, 88, 57, 1, 0),
        (62, 4, 40, 35, flip_none),
        (62, 8, 0, 0, flip_none),
        ),
    (
        (2, anim_continue, 88, 57, 1, 1),
        (62, 5, 39, 34, flip_none),
        (62, 6, 0, 0, flip_none),
        ),
    (
        (2, anim_continue, 88, 57, 4, 1),
        (62, 5, 36, 34, flip_none),
        (62, 7, 0, 0, flip_none),
        ),
    (
        (2, anim_continue, 88, 57, 2, 0),
        (62, 5, 38, 35, flip_none),
        (62, 8, 0, 0, flip_none),
        ),
    )
    
# tim_temim_part_info: Contains information for TIM and TEMIM parts
#     used by the level viewer for these game versions.
# Meaning/type/format for each item:
# 0: part number
# 1: part name
# 2: appears first in this version (0 - TIM demo, 1 - TIM, 2 - TEMIM, 
#     3 - "special")
# 3: x shift
# 4: y shift
# 5: optional x flip axis (ignored for positive items in last tuple)
# 6: optional y flip axis
# 7: tuple containing relevant IDs either TIM BMP files (0 or positive
#     values) or in info_frame_sub_image_pos (negative numbers, the real
#     index is index_first - number - 1 where index_first is looked up in
#     info_frame_sub_image_pos using the part number, only the data for
#     first animation frame is checked). Contains only the numbers that
#     are relevant for displaying the part in a "stopped" state of machine.
# Note: [created] indicated a part which is not present in puzzles in
#   their starting state, but created while the machine is running, e.g.
#   a gun fires a bullet. This term is found in a TIM 2 resource file.
# TODO: unlike in info_frame_sub_image_pos, no full game/demo distinction

# NOTE: x/y shift for Gun part changed (was: -15, -12)

tim_temim_part_info = (
    ( 0, 'Bowling ball', 0, 0, 0, 0, 0, (0,)),
    ( 1, 'Brick wall', 0, 0, 0, 0, 0, (0, 1, 2, 3, 4, 5, 6, 7)), 
    ( 2, 'Wood incline', 0, 0, 0, 0, 0, (0, 1, 2, 3)),
    ( 3, 'Teeter-totter', 0, 0, 0, 0, 0, (0, 2)),
    ( 4, 'Balloon', 0, 0, 0, 0, 0, (0,)),
    ( 5, 'Conveyor belt', 0, 0, 0, 0, 0, (0, 7, 14, 21, 28)),
    ( 6, 'Mouse cage', 0, 0, 0, 0, 0, (0,)),
    ( 7, 'Pulley', 0, 0, 0, 0, 0, (0, 1, 2, 3)),
    ( 8, 'Belt', 0, 0, 0, 0, 0, (0,)),
    ( 9, 'Basketball', 0, 0, 0, 0, 0, (0,)),
    (10, 'Rope', 0, 0, 0, 0, 0, (0,)),
    (11, 'Cage', 0, 0, 0, 0, 0, (0,)),
    (12, 'Pokey Cat', 0, 0, 0, 0, 0, (0,)),
    (13, 'Jack-in-the-box', 1, -6, -59, 32, 29, (-1,)),
    (14, 'Gear', 0, 0, 0, 0, 0, (0,)),
    (15, 'Aquarium', 1, -28, 0, 48, 48, (-1,)),
    (16, 'Bellows', 1, 0, 0, 0, 0, (0,)),
    (17, 'Bucket', 1, 0, 0, 0, 0, (0,)),
    (18, 'Cannon', 1, -9, -46, 64, 52, (-1,)),
    (19, 'Dynamite', 1, 0, 0, 0, 0, (0,)),
    (20, 'Bullet [created]', 1, 0, 0, 0, 0, (0,)),
    (21, 'Electric switch & outlet', 1, 0, 0, 48, 32,
        (-1, -2, -3, -4, -5, -6, -7, -8)),
    (22, 'Dynamite with plunger', 1, 0, 0, 135, 47, (-1,)),
    (23, 'Eye hook', 1, 0, 0, 0, 0, (0,)),
    (24, 'Electric fan', 1, 0, 0, 32, 32, (-1,)),
    (25, 'Flashlight', 1, 0, 0, 0, 0, (0,)),
    (26, 'Generator', 1, 0, -6, 72, 32, (-1, -2, -3, -4)),
    (27, 'Gun', 1, 0, 0, 0, 0, (0,)),
    (28, 'Baseball', 1, 0, 0, 0, 0, (0,)),
    (29, 'Light bulb', 1, -8, -18, 32, 32, (-1, -1, -3)),  #  HACK
    (30, 'Magnifying glass', 1, 0, 0, 0, 0, (0,)), 
    (31, 'Monkey exercise bike', 0, 0, 0, 92, 79, (-1,)),
    (32, 'Pumpkin', 1, 0, 0, 0, 0, (0,)),
    (33, 'Heart-shaped balloon', 1, 0, 0, 0, 0, (0,)),
    (34, 'Christmas tree', 1, 0, 0, 0, 0, (0,)),
    (35, 'Boxing glove', 1, 0, 0, 0, 0, (0,)),
    (36, 'Rocket', 1, -2, -3, 16, 52, (-1,)),
    (37, 'Scrissors', 1, -2, 0, 40, 32, (-1,)),
    (38, 'Solar panels', 1, 0, 0, 72, 32, (-1, -2, -4, -4)),
    (39, 'Trampoline', 1, 0, -3, 48, 28, (-1,)),
    (40, 'Windmill', 1, 0, 0, 0, 0, (0,)),
    (41, 'Explosion [created]', 1, 0, 0, 0, 0, (0,)),
    (42, 'Mort Mouse', 1, 0, 0, 0, 0, (0,)),
    (43, 'Cannon ball', 1, 0, 0, 0, 0, (0,)),
    (44, 'Tennis ball', 1, 0, 0, 0, 0, (0,)),
    (45, 'Candle', 1, 0, 0, 32, 32, (0,)),  #  0 instead of -1 !
    (46, 'Pipe wall', 0, 0, 0, 0, 0, (0, 1, 2, 3, 4, 5, 6, 7)),
    (47, 'Curved pipe wall', 0, 0, 0, 0, 0, (0, 1, 2, 3)),
    (48, 'Wood wall', 0, 0, 0, 0, 0, (0, 1, 2, 3, 4, 5, 6, 7)),
    # Note: part #49 has no graphics, the name is a guess based on
    #   TIM 2 resource file INFO.RES
    (49, 'Rope end [created]', 0, 0, 0, 0, 0, (0,)),
    (50, 'Electric engine', 1, 0, 0, 0, 0, (0,)),
    (51, 'Vacuum', 2, -3, -4, 50, 50, (-1,)),
    (52, 'Cheese', 2, 0, 0, 0, 0, (0,)),
    (53, 'Nail', 2, 0, 0, 0, 0, (0,)),
    (54, 'Mel Schlemming', 2, 0, 0, 0, 0, (0,)),
    # TODO, part #55: item 1 is only available in TEMIM
    (55, '(Title screen logo 1)', 3, 0, 0, 0, 0, (0, 1)), 
    (56, '(Title screen logo 2)', 3, 0, 0, 0, 0, (0, 1, 2, 3, 4, 5, 6)),
    (57, '(Credits screen)', 3, 0, 0, 0, 0, tuple(xrange(14))),
    (58, "Mel's house", 2, 0, 0, 0, 0, (0,)),
    (59, 'Super ball', 2, 0, 0, 0, 0, (0,)),
    (60, 'Dirt wall', 2, 0, 0, 0, 0, (0, 1, 2, 3, 4, 5, 6, 7)),
    (61, 'Ernie Alligator', 2, 0, -17, 80, 16, (-1,)),
    (62, 'Teapot', 2, -45, -35, 30, 22, (-1,)),
    (63, 'Eight ball', 2, 0, 0, 0, 0, (0,)),
    (64, 'Pinball bumper', 1, 0, 0, 0, 0, (0,)),
    (65, 'Clover', 2, 0, 0, 0, 0, (0,)),
    )
    
# tim_temim_part_appearance_texts: Contains text in tuples for part types
#     in TIM and TEMIM used as description text for the "appearance",
#     will be shown for each part in puzzle on the "File info" tab of the GUI.
# Meaning/type/format for each item:
# 0: part number
# 1: flip_texts: tuple of 2 strings, the first is used for an "unflipped",
#     the second for a flipped part. The same tuple is used for horizontal
#     and vertical flipping.
# 2: appearance_texts: Tuple of strings describing the "appearance" of the
#     part, used by PartGraphicsInfo_T1_SubImageID. Must have the same length
#     as anim_frame_info for the same part.
    
flip_texts_left_right = ('left', 'right')
flip_texts_right_left = ('right', 'left')
flip_texts_clockwise = ('clockwise', 'counter-clockwise')

# Note on the usage of "left" and "right": The use of these terms for
#     horizontally flipped parts is somewhat arbitrary. I have applies
#     the following rules to determine the direction:
# * Direction in which some action happens (boxing glove,
#     electric fan, ...)
# * The direction animals & Mel face
# * The direction to which a ball on a conveyor belt powered by
#     a motor will move
    
tim_temim_part_appearance_texts = (
    (3, None, ('bottom-up', 'top-down')),
    (5, None, tuple('Length %d' % x for x in xrange(2, 7)) ),
    (6, flip_texts_right_left, None),
    (7, None, ('bottom', 'left', 'top', 'right')),
    (12, flip_texts_left_right, None),
    (13, flip_texts_clockwise, None),
    (16, flip_texts_right_left, None),
    (18, flip_texts_right_left, None),
    (19, ('fuse right', 'fuse left'), None),
    (22, ('dynamites right', 'dynamites left'), None),
    (23, ('bottom', 'top'), None),
    (24, flip_texts_right_left, None),
    (25, flip_texts_right_left, None),
    (27, flip_texts_right_left, None),
    # HACK for light bulb (uses 0 and 2 in "appearance" field)
    (29, None, ('bulb top', '(unused)', 'bulb bottom')),
    (30, flip_texts_right_left, None),
    (31, flip_texts_right_left, None),
    (35, flip_texts_left_right, None),
    (37, flip_texts_right_left, None),
    (40, flip_texts_clockwise, None),
    (42, flip_texts_left_right, None),
    (47, None, ('top-right', 'top-left', 'bottom-right', 'bottom-left')),
    (50, flip_texts_left_right, None),
    (51, flip_texts_right_left, None),
    (54, flip_texts_right_left, None),
    (61, flip_texts_left_right, None),
    (62, flip_texts_left_right, None),
    )
    
# TIM and TEMIM music_info info
# Format for each item:
# 0: music/song number
# 1: music/song title
# 2: appears first in this version (0 - TIM, 2 - TEMIM)

# TODO, missing music tiles (names are guesses based on TIM.SX)
#     (Sorry, I am not knowledgeable in these musical styles - knt47)

tim_temim_music_info = (
    (1001, 'TIM', 0),
    (1002, 'Euro', 0),
    (1003, 'Funk', 0),
    (1004, 'Reggae', 0),
    (1005, 'Music 1005', 0),  # ZYDI (?)
    (1006, 'Rock', 0),
    (1007, 'Western', 0),
    (1008, 'Blues', 0),
    (1009, 'Classical', 0),
    (1010, 'Salsa', 0),
    (1011, 'Polka', 0),
    (1012, 'Music 1012', 0),  # CRUZ (?)
    (1013, 'Rock 2', 0),
    (1014, 'Romantic', 0),
    (1015, 'Fusion', 0),
    (1016, 'Metal', 0),
    (1017, 'Caribbean', 2), 
    (1018, 'Music 1018', 2),  # SPLN (?)
    (1019, 'Music 1019', 2),  # MILR (?)
    (1020, 'Jungle', 2), 
    (1021, 'Eastern', 2)
    )
