#!/usr/bin/python3
# coding: utf8

# The Incredible Machine (TIM) level/image resource viewer, version 0.2
# "Extra" module, not covered by the CC0 license of "timgres" package

from __future__ import division, print_function, unicode_literals

import itertools, sys

if sys.version_info[0] != 3:
    raise NotImplementedError('Only Python 3 is supported.')
    # izip = itertools.izip
else:
    izip = zip
    xrange = range

# --- Texts for part flip & "program" for TIM 2 ---

# Note: TIM 2 contains a resource file named "PROGPART.RES", however
#     its contents are not always reliable (e.g "Helium Filed" balloon)
#     (note: this contradicts the game help [and my tests] that the behavior
#     of the four balloon types are perfectly identical)

# prog_ball_data: Contains the "programming" information for the programmable
#     ball. A tuple of 4 dicts, one for each programmable parameter.
#     dict key: the value found in the puzzle file,
#     values: The number on the scale in the program dialog of the game (0-10)

prog_ball_data = (
    {y : x for x, y in enumerate((
        1, 2, 6, 10, 21, 50, 121, 151, 201, 1801, 28000))},  # mass
    {y : x for x, y in enumerate((
        0, 64, 96, 128, 144, 160, 192, 224, 256, 302, 396))},  # elasticity
    {y : x for x, y in enumerate((
        1, 250, 750, 1100, 1200, 1500, 1800,
        3000, 6000, 9000, 11000))},  # density
    {y : x for x, y in enumerate((
        0, 8, 12, 16, 24, 34, 46, 60, 78, 98, 128))},  # friction
    )

# Stored in the "extra_data" filed for programmable ball
#    (see tim_2_part_appearance_texts)

# tim_2_part_appearance_texts: Contains text in tuples and dicts for 
#     each part type in TIM 2. It will be used as description text for the
#     "appearance" of the part. This information will be shown for each part
#     in the puzzle, on the "File info" tab of the GUI.
# Meaning/type/format for each item:
# 0: part number
# 1: flip_texts: tuple of 2 strings, the first is used for an "unflipped",
#     the second for a flipped part. The same tuple is used for horizontal
#     and vertical flipping.
# 2: appearance_text_dict: Dict, key is the part number, value is a string.
#     the description of the "appearance". May be None.
# 3: (optional) behavior_text_dict: Dict, key is the part number, value
#     is a string. the description of the "behavior". May be absent.
# 4: (optional) extra_data: Part specific data, currently used only for the
#    programmable ball and the accelerator pipe.

flip_texts_left_right = ('left', 'right')
flip_texts_right_left = ('right', 'left')
flip_texts_clockwise = ('clockwise', 'counter-clockwise')

# Note: The "middle" state is not present for teeter-totters, as the level
#     viewer only shows the state of the puzzle before the machine is started
teeter_totter_appearance = {0: 'bottom-up', 2: 'top-down'}

conveyor_belt_lengths = {y : 'length %d' % x
                         for x, y in enumerate(xrange(0, 25, 5), 2)}

# Note: The "dictionary merge" operator in Python was introduced only in 3.9,
#     we need to use a less elegant solution here in order to support older
#     Python versions.

pool_ball_appearance = dict(enumerate(
    ['8-ball'] + ['%d-ball' % x for x in xrange(1, 8)] +
    ['%d-ball' % x for x in xrange(9, 16)] + ['cue ball']))

laser_direction = {0 : 'right', 1: 'down', 2: 'left', 3: 'up'}

# Part 108 (leaky bucket): The value o "appearance" field affects
#     the initial mass and leakage rate

leaky_bucket_info = {
    y : 'mass-at-start %d' %x for x, y in enumerate((
        1, 8, 18, 30, 51, 80)) }
    
egg_timer_interval = {y : 'time-interval %d' % x
                      for x, y in enumerate(xrange(0, 24, 3))}

tim_2_part_appearance_texts = (
    (3, None, teeter_totter_appearance),
    (4, None, {0: 'default', 6: 'silver', 13: 'dog-shaped', 18: 'happy face'}),
    (5, None, conveyor_belt_lengths),
    (6, flip_texts_right_left, None),
    (7, None, {0: 'bottom', 1: 'left', 2: 'top', 3: 'right'}),
    (12, flip_texts_right_left, None),
    (13, flip_texts_clockwise, None),
    (16, flip_texts_right_left, None),
    # Note for part 18 (cannon): only the "not fired" animation
    #     frames are labeled here (as the viewer only shows the state of
    #     the puzzle before the machine is started)
    (18, None, {
        0: 'right, low-angle', 30: 'right, horizontal',
        60: 'right, high-angle', 91: 'left, low-angle',
        120: 'left, horizontal', 149: 'left, high-angle'}),
    (19, ('fuse left', 'fuse right'), None),
    (21, None, {0: 'switch up', 4: 'switch down'}),
    (23, ('top', 'bottom'), None),
    (24, flip_texts_left_right, None),
    (25, flip_texts_right_left, None),
    (27, flip_texts_right_left, None),
    (30, flip_texts_right_left, None),
    (31, flip_texts_right_left, None),
    (33, flip_texts_left_right, None),
    (35, flip_texts_left_right, None),
    # Note for part 18 (rocket): only the "not launched" animation
    #     frames are labeled here (see comment at pert 18)
    (36, None, {0: 'up', 41: 'right', 78: 'left'}),
    (37, flip_texts_right_left, None),
    (40, flip_texts_clockwise, None),
    (42, flip_texts_left_right, None),
    (45, flip_texts_right_left, None),
    (47, None, {
        0: 'top-right', 1: 'top-left', 2: 'bottom-right', 3: 'bottom-left'}),
    (50, flip_texts_left_right, None),
    (51, flip_texts_left_right, None),
    (53, None, {0: 'up', 4: 'down', 8: 'left', 12: 'right'}),
    (54, flip_texts_right_left, None, 
        {0: 'standing', 134: 'walking', 151: 'running'}),
    (57, None, {
        0: 'top-right', 1: 'top-left', 2: 'bottom-right', 3: 'bottom-left'}),
    (58, None, {0: 'house', 111: 'log cabin'}),
    (61, flip_texts_left_right, None),
    (62, flip_texts_left_right, None),
    (63, None, pool_ball_appearance),
    (70, None, {0: 'up', 39: 'right', 71: 'left'}),
    (71, None, {
        0: 'aquarium', 1: 'wooden crate', 2: 'basket', 3: 'steel cage',
        4: 'cardboard box'}),
    (73, flip_texts_left_right, None),     
    (75, flip_texts_left_right, None),
    (77, flip_texts_right_left, None),
    (80, None, {0: 'blue', 48: 'green', 95: 'yellow'}),
    (81, flip_texts_right_left, {
        0: 'light toast', 16: 'medium toast', 32: 'datk toast'}),
    (87, None, {
        0: 'blue', 1: 'orange', 2: 'red', 3: 'green', 4: 'yellow',
        5: 'striped/default', 13: 'marble'}, None, prog_ball_data),
    # Part 88 (accelerator pipe): The fileds of dict key tuples:
    #     0: flip bits in flags_2, 1: appearance field
    (88, None, None, None, {
        (0x0, 0): 'down', (0x0, 5): 'right',
        (0x20, 0): 'up', (0x10, 5): 'left'}),
    (89, flip_texts_left_right, None),
    (89, flip_texts_right_left, None),
    (90, ('up', 'down'), None),
    (91, None, laser_direction),
    (92, None, laser_direction),
    (93, None, laser_direction),
    (94, None, {
        0: 'bottom-left', 1: 'top-left', 2: 'top-right', 3: 'bottom-right'}),
    # Note for part 95 (laser mixer): only the "no laser" animation
    #     frames are labeled here (see comment at pert 18)
    (95, None, {0: 'right', 85: 'down', 169: 'left', 256: 'up'}),
    (96, None, {
        0: 'white', 1: 'red', 2: 'green', 3: 'blue', 4: 'violet',
        5: 'yellow', 6: 'cyan'}),
    (108, None, leaky_bucket_info),
    (109, flip_texts_left_right, None),
    # Part 116, pinball flipper, the flip texts corresponds to the "moving"
    #     side of the flipper
    (116, flip_texts_right_left, None), 
    (117, None, {
        0: 'right', 7: 'down-right', 14: 'down', 21: 'down-left',
        28: 'left', 35: 'up-left', 42: 'up', 49: 'up-right'}, None ),
    # Part 118, pool table wall: todo, more informative text
    (118, ('top-or-left', 'bottom-or-right'), None), 
    (119, None, {
        0: 'bottom-left', 1: 'left', 2: 'top-left', 3: 'top',
        4: 'top-right', 5: 'right', 6: 'bottom-right', 7: 'bottom'}),
    (137, flip_texts_right_left, egg_timer_interval),
    (139, None, teeter_totter_appearance),
    )
